@@ -14,23 +14,39 @@
 #######################################################################
 
 package Win32::API;
-use strict;
-use warnings;
-require Exporter;      # to export the constants to the main:: space
-require DynaLoader;    # to dynuhlode the module.
-use vars qw( $DEBUG $sentinal @ISA @EXPORT_OK %Imported $VERSION );
-
-@ISA = qw( Exporter DynaLoader );
-@EXPORT_OK = qw( ReadMemory IsBadReadPtr MoveMemory
-WriteMemory SafeReadWideCString ); # symbols to export on request
-
-use Scalar::Util qw( looks_like_number );
-
-$DEBUG = 0;
-
+    use strict;
+    use warnings;
+    use Config;
 BEGIN {
-sub ERROR_NOACCESS () { 998 }
-eval " *Win32::API::Type::IVSIZE = *Win32::API::More::IVSIZE = *IVSIZE = sub  () { ".length(pack('J',0))." }";
+    require Exporter;      # to export the constants to the main:: space
+    require DynaLoader;    # to dynuhlode the module.
+
+    sub ISCYG ();
+    eval "sub ISCYG () { ".($^O eq 'cygwin' ? 1 : 0)."}";
+    no warnings 'uninitialized';
+    die "Win32::API on Cygwin requires the cygpath tool on PATH"
+        if ISCYG && index(`cygpath --help`,'Usage: cygpath') == -1;
+
+    use vars qw( $DEBUG $sentinal @ISA @EXPORT_OK %Imported $VERSION );
+
+    @ISA = qw( Exporter DynaLoader );
+    @EXPORT_OK = qw( ReadMemory IsBadReadPtr MoveMemory
+    WriteMemory SafeReadWideCString ); # symbols to export on request
+
+    use Scalar::Util qw( looks_like_number );
+
+    $DEBUG = 0;
+    
+    sub ERROR_NOACCESS	() { 998 }
+    sub ERROR_NOT_ENOUGH_MEMORY () { 8 }
+    sub APICONTROL_CC_STD	() { 0 }
+    sub APICONTROL_CC_C	() { 1 }
+    sub APICONTROL_CC_mask  () { 0x7 }
+    sub APICONTROL_UseMI64	() { 0x8 }
+    sub APICONTROL_is_more	() { 0x10 }
+    sub APICONTROL_has_proto() { 0x20 }
+    eval " *Win32::API::Type::PTRSIZE = *Win32::API::More::PTRSIZE = *PTRSIZE = sub  () { ".$Config{ptrsize}." }";
+    eval " *Win32::API::Type::IVSIZE = *Win32::API::More::IVSIZE = *IVSIZE = sub  () { ".$Config{ivsize}." }";
 }
 
 sub DEBUG {
@@ -49,8 +65,6 @@ use File::Basename ();
 #######################################################################
 # STATIC OBJECT PROPERTIES
 #
-$VERSION = '0.70_02';
-
 #### some package-global hash to
 #### keep track of the imported
 #### libraries and procedures
@@ -60,22 +74,25 @@ my %Procedures = ();
 
 #######################################################################
 # dynamically load in the API extension module.
-#
-bootstrap Win32::API;
+# BEGIN required for constant subs in BOOT:
+BEGIN {
+    $VERSION = '0.76_01';
+    bootstrap Win32::API;
+}
 
 #######################################################################
 # PUBLIC METHODS
 #
 sub new {
-    my ($class, $dll, $hproc) = (shift, shift);
+    my ($class, $dll, $hproc, $ccnum, $outnum) = (shift, shift);
     if(! defined $dll){
         $hproc = shift;
     }
     my ($proc, $in, $out, $callconvention) = @_;
-    my ($hdll, $freedll) = (0, 0);
+    my ($hdll, $freedll, $proto, $stackunwind) = (0, 0, 0, 0);
     my $self = {};
     if(! defined $hproc){
-        if ($^O eq 'cygwin' and $dll ne File::Basename::basename($dll)) {
+        if (ISCYG() and $dll ne File::Basename::basename($dll)) {
     
             # need to convert $dll to win32 path
             # isn't there an API for this?
@@ -113,13 +130,13 @@ sub new {
     }
     #### determine if we have a prototype or not, outtype is for future use in XS
     if ((not defined $in) and (not defined $out)) {
-        ($proc, $self->{in}, $self->{intypes}, $self->{out}, $self->{outtype},
-         $self->{cdecl}) = parse_prototype($class, $proc);
+        ($proc, $self->{in}, $self->{intypes}, $outnum, $self->{outtype},
+         $ccnum) = parse_prototype($class, $proc);
         if( ! $proc ){
             Win32::API::FreeLibrary($hdll) if $freedll;
             return undef;
         }
-        $self->{proto} = 1;
+        $proto = 1;
     }
     else {
         $self->{in} = [];
@@ -143,8 +160,8 @@ sub new {
                 } else {undef(@{$self_in});} #empty arr, as if in param was ""
             }
         }
-        $self->{out}   = $class->type_to_num($out, 1);
-        $self->{cdecl} = calltype_to_num($callconvention);
+        $outnum   = $class->type_to_num($out, 1);
+        $ccnum = calltype_to_num($callconvention);
     }
 
     if(!$hproc){ #if not non DLL func
@@ -172,12 +189,34 @@ sub new {
     else {
         DEBUG "Using non-DLL function pointer '$hproc' for '$proc'\n";
     }
-
+    if(PTRSIZE == 4 && $ccnum == APICONTROL_CC_C) {#fold out on WIN64
+        #calculate add to ESP amount, in units of 4, will be *4ed later
+        $stackunwind += $_ == T_QUAD || $_ == T_DOUBLE ? 2 : 1 for(@{$self->{in}});
+        if($stackunwind > 0xFFFF) {
+            DEBUG "FAILED This function has too many parameters (> ~65535) \n";
+            Win32::API::FreeLibrary($hdll) if $freedll;
+            Win32::SetLastError(ERROR_NOT_ENOUGH_MEMORY);
+            $! = Win32::GetLastError();
+            return undef;
+        }
+    }
     #### ok, let's stuff the object
     $self->{procname} = $proc;
     $self->{dll}      = $hdll;
     $self->{dllname}  = $dll;
-    $self->{proc}     = $hproc;
+    $self->{control}  = pack((PTRSIZE == 8 ? 'Q' : 'L').'L'
+                             .(PTRSIZE == 8 ? 'L' : '') #padding for x64
+                             .(PTRSIZE == 8 ? 'Q' : 'L')
+                             .(PTRSIZE == 8 ? 'Q' : 'L')
+                        , $hproc
+                        ,($class eq "Win32::API::More" ? APICONTROL_is_more : 0)
+                        | ($proto ? APICONTROL_has_proto : 0)
+                        | $ccnum
+                        | (PTRSIZE == 8 ? 0 :  $stackunwind << 8)
+                        | $outnum << 24
+                        ,(PTRSIZE == 8 ? (0) : ())
+                        , (($self->{in})+0)
+                        , (exists $self->{intypes} ? ($self->{intypes})+0 : 0));
 
     #### keep track of the imported function
     if(defined $dll){
@@ -186,8 +225,44 @@ sub new {
     }
     DEBUG "Object blessed!\n";
 
+    #one-off classes might go away since there is no measurable time difference
+    #between hv_fetch_ent with a HEK SV and this crazy scheme
+    #
+    #there are unnecessery pack/unpack calls here
+    my ($packedcontrol, $controlmask) = (pack('p',$self->{control}), '');
+    my @splitcontrol = split '', $packedcontrol;
+    #this probably isn't the most efficient algorithm for generating an
+    #null-less XOR key/mask that results in a null-less delta
+    foreach my $char (@splitcontrol){
+        my ($intchar, $i) = unpack('C', $char);
+        for($i = 1; $i < 256; $i++){
+            if(($intchar ^ $i) != 0) {
+                $controlmask .= pack('C', $i);
+                goto chardone;
+            }
+        }
+        #couldn't find a non null byte to XOR original
+        die "Win32::API: can't make XOR mask";
+        chardone:
+        0;
+    }
+
+    $packedcontrol = pack((PTRSIZE == 8 ? 'Q' : 'L'),
+         unpack((PTRSIZE == 8 ? 'Q' : 'L'), $packedcontrol)
+         ^ unpack((PTRSIZE == 8 ? 'Q' : 'L'), $controlmask))
+         .$controlmask;
+    die "Win32::API: can't make XOR mask" if index($packedcontrol, "\x00") != -1;
+    $packedcontrol = "W32::API".$packedcontrol; #"W32::API" is exactly 8 bytes
+    {
+        no strict 'refs';
+        @{$packedcontrol.'::ISA'} = $class;
+        #since Call() is the most frequently called function, put it in the
+        #one off stash, this is to take advantage of a shortcut in S_method_common
+        *{$packedcontrol.'::Call'} = *Win32::API::Call;
+    }
     #### cast the spell
-    bless($self, $class);
+    bless($self, $packedcontrol);
+    #bless($self, $class);
     return $self;
 }
 
@@ -218,6 +293,17 @@ sub DESTROY {
         Win32::API::FreeLibrary($Libraries{$self->{dllname}});
         delete($Libraries{$self->{dllname}});
     }
+    #can't clog up symbol table, delete our unique class name
+    my $class = ref($self).'::';
+    {
+        no strict 'refs';
+        #wipe @ISA, etc
+        foreach my $symbol (keys %{$class}) {
+            delete ${$class}{$symbol};
+        }
+        #wipe package
+        delete $::{$class};
+    }
 }
 
 # Convert calling convention string (_cdecl|__stdcall)
@@ -226,15 +312,16 @@ sub DESTROY {
 sub calltype_to_num {
     my $type = shift;
 
-    if (!$type || $type eq "__stdcall") {
-        return 0;
+    if (!$type || $type eq "__stdcall" || $type eq "WINAPI" || $type eq "NTAPI"
+        || $type eq "CALLBACK"  ) {
+        return APICONTROL_CC_STD;
     }
-    elsif ($type eq "_cdecl" || $type eq "__cdecl") {
-        return 1;
+    elsif ($type eq "_cdecl" || $type eq "__cdecl" || $type eq "WINAPIV") {
+        return APICONTROL_CC_C;
     }
     else {
         warn "unknown calling convention: '$type'";
-        return 0;
+        return APICONTROL_CC_STD;
     }
 }
 
@@ -256,7 +343,7 @@ sub type_to_num {
         or $type eq 'n'
         or $type eq 'l'
         or $type eq 'L'
-        or ( IVSIZE == 8  and $type eq 'Q' || $type eq 'q'))
+        or ( PTRSIZE == 8  and $type eq 'Q' || $type eq 'q'))
     {
         $num = 1;
     }
@@ -285,7 +372,7 @@ sub type_to_num {
     {
         $num = 6;
     }
-    elsif (IVSIZE == 4 and $type eq 'q' || $type eq 'Q')
+    elsif (PTRSIZE == 4 and $type eq 'q' || $type eq 'Q')
     {
         $num = 5;
     }
@@ -341,7 +428,7 @@ sub type_to_num {
         or $type eq 'n'
         or $type eq 'l'
         or $type eq 'L'
-        or ( IVSIZE == 8  and $type eq 'Q' || $type eq 'q')
+        or ( PTRSIZE == 8  and $type eq 'Q' || $type eq 'q')
         or (! $out and  # in XS short 'in's are interger/numbers code
             $type eq 'S'
             || $type eq 's'))
@@ -383,7 +470,7 @@ sub type_to_num {
             $num |= 0x80;
         }
     }
-    elsif (IVSIZE == 4 and $type eq 'q' || $type eq 'Q')
+    elsif (PTRSIZE == 4 and $type eq 'q' || $type eq 'Q')
     {
         $num = 5;
         if(defined $out && $type eq 'Q'){
@@ -534,7 +621,27 @@ sub parse_prototype {
     }
 }
 
-
+#
+# XXX hack, see the proper implementation in TODO
+# The point here is dont let fork children free the parent's DLLs.
+# CLONE runs on ::API and ::More, thats bad and causes a DLL leak, make sure
+# CLONE dups the DLL handles only once per CLONE
+# GetModuleHandleEx was not used since that is a WinXP and newer function, not Win2K.
+# GetModuleFileName was used to get full DLL pathname incase SxS/multiple DLLs
+# with same file name exist in the process. Even if the dll was loaded as a
+# relative path initially, later SxS can load a DLL with a different full path
+# yet same file name, and then LoadLibrary'ing the original relative path
+# might increase the refcount on the wrong DLL or return a different HMODULE
+sub CLONE { 
+    return if $_[0] ne "Win32::API";
+    
+    _my_cxt_clone();
+    foreach( keys %Libraries){
+        if($Libraries{$_} != Win32::API::LoadLibrary(Win32::API::GetModuleFileName($Libraries{$_}))){
+            die "Win32::API::CLONE unable to clone DLL \"$Libraries{$_}\" Unicode Problem??";
+        }
+    }
+}
 
 1;
 
@@ -738,7 +845,8 @@ The type of the value returned by the function.
 
 =item 6.
 And optionally you can specify the calling convention, this defaults to
-'__stdcall', alternatively you can specify '_cdecl' or '__cdecl' (API > v0.68).
+'__stdcall', alternatively you can specify '_cdecl' or '__cdecl' (API > v0.68)
+or (API > v0.70_02) 'WINAPI', 'NTAPI', 'CALLBACK' (__stdcall), 'WINAPIV' (__cdecl) .
 False is __stdcall. Vararg functions are always cdecl. MS DLLs are typically
 stdcall. Non-MS DLLs are typically cdecl.
 
@@ -860,8 +968,6 @@ value is a unsigned 64 bit integer number (unsigned long long, unsigned __int64)
 See next item for details.
 
 =item C<q>:
-This function is experimental and subject to change.
-
 value is a signed 64 bit integer number (long long, __int64)
 If your perl has 'Q'/'q' quads support for L<perlfunc/pack> then Win32::API's 'q'
 is a normal perl numeric scalar. All 64 bit Perls have quad support. Almost no
@@ -870,8 +976,12 @@ Win32::API's 'q'/'Q' letter is a packed 8 byte string. So C<0x8000000050000000>
 from a perl with native Quad support would be written as
 C<"\x00\x00\x00\x50\x00\x00\x00\x80"> on a 32 bit Perl without Quad support.
 To improve the use of 64 bit integers with Win32::API on a 32 bit Perl without
-Quad support, there is a per Win32::API object setting called L</UseMI64>
+Quad support, there is a per Win32::API::* object setting called L</UseMI64>
 that causes all quads to be accepted as, and returned as L<Math::Int64> objects.
+For "in" params in Win32::API and Win32::API::More and "out" in
+Win32::API::Callback only, if the argument is a reference, it will automatically
+be treated as a Math::Int64 object without having to previously call
+L</UseMI64>.
 
 =item C<F>: 
 value is a floating point number (float)
@@ -1059,7 +1169,7 @@ done to vaildate the argument's type vs the parameter's type if the function
 has a C prototype definition (not letter definition). First, if the parameter
 type starts with the LP prefix, the LP prefix is stripped, then compared to
 the argument's type. If that fails, the Win32::API::Type database
-(see L<Win32::API::Type\typedef>)
+(see L<Win32::API::Type/typedef>)
 will be used to convert the parameter type to the base type. If that fails,
 the parameter type will be stripped of a trailing whitespace then a '*', and
 then checked against the base type. L<Dies|perlfunc/die> if the parameter and
@@ -1132,8 +1242,6 @@ on this function of the same name.
 
 =head3 SafeReadWideCString
 
-This function is experimental and subject to change.
-
     $source = Encode::encode("UTF-16LE","Just another perl h\x{00E2}cker\x00");
     $string = SafeReadWideCString(unpack('J',pack('p', $source)));
     die "impossible" if $source ne "Just another perl h\x{00E2}cker";
@@ -1165,8 +1273,6 @@ The main method of a Win32::API object. Documented elsewhere in this document.
 
 =head3 UseMI64
 
-This function is experimental and subject to change.
-
     $bool = $APIObj->UseMI64();
     $t_or_f_of_newbool = $APIObj->UseMI64($newbool);
 
@@ -1176,15 +1282,30 @@ before calling UseMI64. Win32::API does not C<use> Math::Int64 for you.
 Works on Win32::API and Win32::API::Callback objects. This method
 does not exist if your Perl natively supports Quads (64 bit Perl for example).
 Takes 1 optional parameter, which is a true or false value to use or don't use
-Math::Int64, returns the new setting, which is a true or false value. If called
+Math::Int64, returns the old setting, which is a true or false value. If called
 without any parameters, returns current setting, which is a true or false value,
 without setting the option. As discussed in L</q>, if your not using Math::Int64
-you must supply/will receive 8 byte scalar strings for quads.
+you must supply/will receive 8 byte scalar strings for quads. For "in" params
+in Win32::API and Win32::API::More and "out" in Win32::API::Callback only,
+if the argument is a reference, it will automatically be treated as a
+Math::Int64 object without having to previously call this function.
 
 =head1 HISTORY
 
 =over 4
 
+=item UseMI64 API change
+
+Starting in 0.71, UseMI64 on a set returns old value, not previously
+new value.
+
+=item fork safe
+
+Starting in 0.71, a Win32::API object can go through a fork and work
+correctly in the child and parent psuedo-processes. Previously when either
+psuedo-processes exited, the DLL would be unloaded and the other
+psuedo-processes would crash if a Call() was done on the object.
+
 =item return value signedness
 
 Prior to 0.69, for numeric integer types, the return scalar was always signed.
@@ -1226,11 +1347,15 @@ See the C<Changes> file for more details, many of which not mentioned here.
 
 =over 4
 
+=item E<nbsp> Unicode DLL paths
+
+Untested.
+
 =item E<nbsp> 32 bit perls with native quads
 
 Untested.
 
-=item E<nbsp> ithreads/fork/cloning
+=item E<nbsp> ithreads
 
 Untested.
 
@@ -1250,6 +1375,8 @@ L<Win32::API::Type>
 
 L<Win32::API::Callback>
 
+L<Win32::API::Callback::IATPatch>
+
 L<http://homepage.ntlworld.com/jonathan.deboynepollard/FGA/function-calling-conventions.html>
 
 =head1 AUTHOR
