@@ -16,7 +16,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION @ISA $Stage2FuncPtrPkd );
 
-$VERSION = '0.70_02';
+$VERSION = '0.76_01';
 
 
 require Exporter;      # to export the constants to the main:: space
@@ -42,13 +42,22 @@ use Config;
 BEGIN {
     #there is supposed to be 64 bit IVs on 32 bit perl compatibility here
     #but it is untested
-    eval "sub IVSIZE () { ".length(pack('J',0))." }";
+    #Win64 added in 5.7.3
+    eval "sub IVSIZE () { ".length(pack($] >= 5.007003 ? 'J' : 'I' ,0))." }";
     #what kind of stack processing/calling convention/machine code we needed
     eval "sub ISX64 () { ".(index($Config{'archname'},"MSWin32-x64") == 0 ?  1 : 0)." }";
+    eval 'sub OPV () {'.$].'}';
+    sub OPV();
     sub CONTEXT_XMM0();
     sub CONTEXT_RAX();
-    *UseMI64 = *Win32::API::UseMI64; #keep UseMI64 out of export list
     *IsBadStringPtr = *Win32::API::IsBadStringPtr;
+    sub PTRSIZE ();
+    *PTRSIZE = *Win32::API::PTRSIZE;
+    sub PTRLET ();
+    *PTRLET = *Win32::API::Type::pointer_pack_type;
+    if(OPV <= 5.008000){ #dont have unpackstring in C
+        eval('sub _CallUnpack {return unpack($_[0], $_[1]);}');
+    }
 }
 #######################################################################
 # dynamically load in the API extension module.
@@ -83,7 +92,7 @@ sub new {
             $self->{inbytes} += 8; #always 8
         }
         else{
-            $self->{inbytes} += IVSIZE; #4 or 8
+            $self->{inbytes} += PTRSIZE; #4 or 8
         }
     }
     $self->{outtype} = $out;
@@ -110,7 +119,7 @@ sub MakeStruct {
     return $struct;
 }
 
-#this was rewritten in XS
+#this was rewritten in XS, and is broken b/c it doesn't work on 32bit Perl with Quads
 #sub MakeParamArr { #on x64, never do "$i++;       $packedparam .= $arr->[$i];"
 #    #on x86, structs and over word size params appears on the stack,
 #    #on x64 anything over the size of a "word" is passed by pointer
@@ -175,14 +184,19 @@ sub MakeStruct {
 if(! ISX64 ) {
 *RunCB  = sub {#32 bits
     my $self = $_[0];
-    my (@pass_arr, $return, $typeletter, $inbytes);
+    my (@pass_arr, $return, $typeletter, $inbytes, @arr);
     $inbytes = $self->{inbytes};
     #first is ebp copy then ret address
-    $inbytes += IVSIZE * 2;
-    my $paramcount = $inbytes / IVSIZE ;
-    my $stackstr = unpack('P['.$inbytes.']', pack('J', $_[1]));
-    my @arr = unpack("(a[J])[$paramcount]",$stackstr);
-    #print Dumper(\@arr);
+    $inbytes += PTRSIZE * 2;
+    my $paramcount = $inbytes / PTRSIZE ;
+    my $stackstr = unpack('P'.$inbytes, pack(PTRLET, $_[1]));
+    #pack () were added in 5.7.2
+    if (OPV > 5.007002) {
+        @arr = unpack("(a[".PTRLET."])[$paramcount]",$stackstr);
+    } else {
+        #letter can not be used for size, must be numeric on 5.6
+        @arr = unpack(("a4") x $paramcount,$stackstr);
+    }
     shift @arr, shift @arr; #remove ebp copy and ret address
     $paramcount -= 2;
     $return = &{$self->{sub}}(@{MakeParamArr($self, \@arr)});
@@ -196,16 +210,21 @@ if(! ISX64 ) {
     if($typeletter eq 'n' || $typeletter eq 'N'
           || $typeletter eq 'l' || $typeletter eq 'L'
           || $typeletter eq 'i' || $typeletter eq 'I'){
-        $_[2] = pack('J', $return);
+        $_[2] = pack(PTRLET, $return);
     }elsif($typeletter eq 'q' || $typeletter eq 'Q'){
-        if($self->{'UseMI64'}){ #un/signed meaningless
-            $_[2] = Math::Int64::int64_to_native($return);
+        if(IVSIZE == 4){
+            if($self->{'UseMI64'} || ref($return)){ #un/signed meaningless
+                $_[2] = Math::Int64::int64_to_native($return);
+            }
+            else{
+                warn("Win32::API::Callback::RunCB return value for return type Q is under 8 bytes long")
+                if length($return) < 8;
+                $_[2] = $return.''; #$return should be a 8 byte string
+                #will be garbage padded in XS if < 8, but must be a string, not a IV or under
+            }
         }
         else{
-            warn("Win32::API::Callback::RunCB return value for return type Q is under 8 bytes long")
-            if length($return) < 8;
-            $_[2] = $return.''; #$return should be a 8 byte string
-            #will be garbage padded in XS if < 8, but must be a string, not a IV or under
+            $_[2] = pack($typeletter, $return);
         }
     }elsif($typeletter eq 'f' || $typeletter eq 'F' ){
         $_[2] = pack('f', $return);
@@ -213,11 +232,11 @@ if(! ISX64 ) {
         $_[2] = pack('d', $return);
         $_[4] = 1; #use double
     }else { #return null
-        $_[2] = pack('J', 0);
+        $_[2] = "\x00" x 8;
     }
     
     if(! $self->{cdecl}){
-        $_[3] = IVSIZE * $paramcount; #stack rewind amount in bytes
+        $_[3] = PTRSIZE * $paramcount; #stack rewind amount in bytes
     }
     else{$_[3] = 0;}
 };
@@ -320,7 +339,7 @@ sub MakeCB{
 
 
     #begin x64 part
-    #these packs dont constant fold :-(
+    #these packs dont constant fold in < 5.17 :-(
     #they are here for readability
     :(''.pack('C', 0b01000000 #REX base
             | 0b00001000 #REX.W
@@ -332,6 +351,16 @@ sub MakeCB{
     ).pack('C', 0xB8) #mov to rax register
     .$Stage2FuncPtrPkd
     ."\xFF\xE0");#       jmp     rax
+#making a full function in Perl in x64 was removed because RtlAddFunctionTable
+#has no effect on VS 2008 debugger, it is a bug in VS 2008, in WinDbg the C callstack
+#is correct with RtlAddFunctionTable, and broken without RtlAddFunctionTable
+#in VS 2008, the C callstack was always broken since WinDbg and VS 2008 both
+#*only* use Unwind Tables on x64 to calculate C callstacks, they do not, I think,
+#use 32 bit style EBP/RBP walking, x64 VC almost never uses BP addressing anyway.
+#The easiest fix was to not have dynamic machine code in the callstack at all,
+#which is what I did. Having good C callstacks in a debugger with ::API and
+#::Callback are a good goal.
+#
 ##--- c:\documents and settings\administrator\desktop\w32api\callback\callback.c -
 #    $code .= "\x4C\x8B\xDC";#         mov         r11,rsp
 #    $code .= "\x49\x89\x4B\x08";#      mov         qword ptr [r11+8],rcx 
@@ -480,7 +509,10 @@ that causes all quads to be accepted as, and returned as L<Math::Int64>
 objects. 4 to 8 byte long pass by copy/return type C aggregate types
 are very rare in Windows, but they are supported as "in" and return
 types by using 'q'/'Q' on 32 and 64 bits. Converting between the C aggregate
-and its representation as a quad is upto the reader.
+and its representation as a quad is upto the reader. For "out" in
+Win32::API::Callback (not "in"), if the argument is a reference, it will
+automatically be treated as a Math::Int64 object without having to
+previously call this function.
 
 =item C<F>: 
 value is a floating point number (float)
@@ -520,7 +552,8 @@ parameter will somehow indiciate how many additional stack parameters you are
 receiving. The Ns in @_ will eventually become garbage, technically they are
 the return address, saved registers, and C stack allocated variables of the
 caller. They are effectivly garbage for your vararg callback. All vararg
-callbacks on 32 bits must supply a calling convention, and it must be '__cdecl'.
+callbacks on 32 bits must supply a calling convention, and it must be '__cdecl'
+or 'WINAPIV'.
 
 =head2 METHODS
 
@@ -528,6 +561,10 @@ callbacks on 32 bits must supply a calling convention, and it must be '__cdecl'.
 
 See L<Win32::API/UseMI64>.
 
+=head1 SEE ALSO
+
+L<Win32::API::Callback::IATPatch>
+
 =head1 AUTHOR
 
 Aldo Calpini ( I<dada@perl.it> ).
